from unittest.mock import patch

import pytest

from gso.products.product_types.edge_port import EdgePort
from test.services.conftest import MockedNetboxClient
from test.workflows import (
    assert_complete,
    assert_lso_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
@patch("gso.services.netbox_client.NetboxClient.get_interface_by_name_and_device")
def test_validate_edge_port_success(
    mock_get_interface_by_name_and_device,
    mock_execute_playbook,
    edge_port_subscription_factory,
    faker,
    data_config_filename,
):
    subscription_id = edge_port_subscription_factory()
    mock_get_interface_by_name_and_device.side_effect = [
        MockedNetboxClient.BaseMockObject(
            name="iFace1",
            module=MockedNetboxClient.BaseMockObject(display="display1"),
            description=subscription_id,
            enabled=True,
        ),
        MockedNetboxClient.BaseMockObject(
            name="iFace2",
            module=MockedNetboxClient.BaseMockObject(display="display2"),
            description=subscription_id,
            enabled=True,
        ),
        MockedNetboxClient.BaseMockObject(
            name="Iface3",
            module=MockedNetboxClient.BaseMockObject(display="display3"),
            description=subscription_id,
            enabled=True,
        ),
    ]

    #  Run workflow
    initial_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow("validate_edge_port", initial_data)

    state = extract_state(result)
    subscription_id = state["subscription_id"]

    for _ in range(1):
        result, step_log = assert_lso_success(result, process_stat, step_log)

    assert_complete(result)
    subscription = EdgePort.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert mock_execute_playbook.call_count == 1
    # One time for getting the LAG and two times for getting the interfaces
    assert mock_get_interface_by_name_and_device.call_count == 3
